
/*
 * H.323 protocol handler
 *
 * Copyright (c) 2002-2003 File-Sure Pte. Ltd.
 *
 * The contents of this file are distributed under the terms of
 * the GNU General Public License
 *
 * Software distributed under this License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Original Code is File-Sure Pte. Ltd.
 * -----------------------------------------------------------------
 * $Log: chan_h323.h,v $
 * Revision 1.4  2002/05/18 01:08:36  benjamin
 * This release tested with Zapata XP100 FXO card.
 *
 * Revision 1.3  2002/04/16 09:23:41  benjamin
 * This records another merge with lastest Asterisk CVS.
 * Some new files still missing.
 *
*/

#ifndef _chan_h323_H
#define _chan_h323_H

extern "C" {
#include <stdio.h>
#include <pthread.h>
#include <string.h>
#include <asterisk/channel.h>
#include <asterisk/channel_pvt.h>
#include <asterisk/config.h>
#include <asterisk/logger.h>
#include <asterisk/module.h>
#include <asterisk/pbx.h>
#include <asterisk/options.h>
#include <asterisk/frame.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/times.h>
#include <errno.h>
#include <unistd.h>
#include <stdlib.h>
#include <arpa/inet.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/fd.h>

char *ast_extension_data(struct ast_channel *c, char *context,
                char *ext, int priority, char *callerid);
void h323_caps_notify(struct ast_channel *c, int caps[2]);

void P(int sid);
void V(int sid);
int semtran(int key);
int *create_alarm(int delay);
}

#define H323MAGIC	0x8129abcd
#define PHONE_MAX_BUF 	480
#define DEFAULT_GAIN 	0x100
#define NBUF		6
#define MAXREADLEN      PHONE_MAX_BUF * NBUF
#define MAXWRITELEN     PHONE_MAX_BUF * NBUF
#define H323_OFFHOOK	0x0111
#define H323_DTMF 	0x0112
#define MAJOR_VERSION	1
#define MINOR_VERSION	0
#define BUILD_TYPE	BetaCode
#define BUILD_NUMBER	12
#define WRITESEM	0x121
#define MAX_PWCAPS	5
#define debugtrace	option_verbose

class GatewayConnection;


struct h323_pvt
{
   unsigned char magic[2];
   int  infd[2];			// input pipe
   struct ast_channel *owner;	// channel we are attached
   int  lastoutput;
   int  lastinput;
   char dev[128];		// device name
   char devcalled[5];		// type of device called
   struct ast_frame fr;		// frame for read to return
   int  ibuflen;		// input buffer marker
   int  obuflen;		// output buffer marker
   int  isactive;
   int  silencesuppression;
   int  incomingCall;
   char context[AST_MAX_EXTENSION];
   unsigned char obuf[MAXREADLEN];
   unsigned char ibuf[MAXWRITELEN];
   char ext[AST_MAX_EXTENSION];
   char callerid[AST_MAX_EXTENSION];
   char language[MAX_LANGUAGE];
   char dtmf_str[AST_MAX_EXTENSION];
   char frbuf[480*6];
   int  dtmf_ontime;
   int  dtmf_offtime;
   int  max_waitans_tm;
   GatewayConnection *connection;
   int  vexception;		// indicates exception type to h323_exception
   char *type;
   pthread_mutex_t readlock;
   pthread_mutex_t writelock;
   pthread_mutex_t watermark;
   pthread_mutex_t pnxlock;
   int	writesem;
   int  *waitsem;
   int  frames_buffered;
   int  in_buffered;
   int  rx_frlen;
   int  tx_frlen;
   int  rx_framerate;
   int  tx_framerate;
   int  in_framerate;
   int  out_framerate;
   int  maxwritelen;
   int  maxreadlen;
   int  readwait;
   struct CodecXRefInfo  *rx_codec;
   struct CodecXRefInfo  *tx_codec;
   struct h323_pvt	*next; // link used for list mgmt
};

extern "C" 
{
   struct h323_pvt *init_h323_pvt(const char *devnm);
   struct ast_channel *h323_new(struct h323_pvt *i,int state, char *context, int     format);
   int h323_hangup(struct ast_channel *ast);
   int h323_answer(struct ast_channel *ast);
   int h323_digit(struct ast_channel *ast, char digit);
   struct ast_channel *h323_request(char *type, int format, void *data);
   int h323_call(struct ast_channel *ast, char *dest, int timeout);
   struct ast_frame *h323_exception(struct ast_channel *ast);
   struct ast_frame  *h323_read(struct ast_channel *ast);
   int h323_write_buf(struct h323_pvt *p, char *buf, int len, int frlen);
   int h323_write(struct ast_channel *ast, struct ast_frame *frame);
}


#include <ptclib/ipacl.h>

#include <h323.h>


class OpalLineInterfaceDevice;
class OpalPnXDevice;

class GatewayEndPoint : public H323EndPoint
{
  PCLASSINFO(GatewayEndPoint, H323EndPoint);

  public:
    GatewayEndPoint();
    ~GatewayEndPoint();

    // overrides from H323EndPoint
    H323Connection * CreateConnection(unsigned callReference);
    BOOL OnIncomingCall(H323Connection &, const H323SignalPDU &, 
							H323SignalPDU &);
    H323Connection::AnswerCallResponse OnAnswerCall(
      H323Connection & connection,    /// Connection that was established
      const PString & callerName,       /// Name of caller
      const H323SignalPDU & setupPDU,   /// Received setup PDU
      H323SignalPDU & connectPDU        /// Connect PDU to send. 
    );

    unsigned NewSessId() {  return sessid++; }
    
    void OnConnectionEstablished(H323Connection & connection, const PString &);
    void OnConnectionCleared(H323Connection & connection, const PString &);
    BOOL OpenAudioChannel(H323Connection &, BOOL, unsigned, H323AudioCodec &);

    // new functions
    BOOL Initialise(void);

    virtual PString RewriteE164(const PString &dialedDigits);
    virtual PString RewriteE164(const PString &dialedDigits, 
					const PStringToString &myplan);

    static const char	*type;
    static const char	*tdesc;
    struct h323_pvt	*pvthd;

  protected:

    void SetEndpointTypeInfo( H225_EndpointType & info ) const;
    PStringArray SupportedPrefix;

    PIpAccessControlList accessControl;

    unsigned jitter;
    unsigned sessid;
    friend class GatewayConnection;
};

class OpalPnXDevice : public OpalLineInterfaceDevice
{
  PCLASSINFO(OpalPnXDevice, OpalLineInterfaceDevice);

  public:
    struct h323_pvt 	*pvt;

    OpalPnXDevice(h323_pvt *p) { pvt = p; Open(p->dev); }
    ~OpalPnXDevice() { Close(); }

    virtual BOOL Open(
      const PString & device  /// Device identifier name.
    );

    virtual BOOL Close();

    virtual PString GetName() const;
    virtual unsigned GetLineCount();
    /**Determine if line is currently off hook.
       This returns TRUE if GetLineState() is a state that implies the line is
       off hook (eg OffHook or LineBusy).
      */
    virtual BOOL IsLineOffHook(
      unsigned line   /// Number of line
    );
    /**Set the state of the line.
       Note that not be possible on a given line.
      */
    virtual BOOL SetLineOffHook(
      unsigned line,        /// Number of line
      BOOL newState = TRUE  /// New state to set
    );

    /**Get the media formats this device is capable of using.
      */
    virtual OpalMediaFormat::List GetMediaFormats() const;

   /**Stop the read codec.
      */
    virtual BOOL StopReadCodec(
      unsigned line   /// Number of line
    );

    /**Stop the write codec.
      */
    virtual BOOL StopWriteCodec(
      unsigned line   /// Number of line
    );

    /**Set the codec for reading.
      */
    virtual BOOL SetReadFormat(
      unsigned line,    /// Number of line
      const OpalMediaFormat & mediaFormat   /// Codec type
    );
    /**Set the codec for writing.
      */
    virtual BOOL SetWriteFormat(
      unsigned line,    /// Number of line
      const OpalMediaFormat & mediaFormat   /// Codec type
    );

    /**Get the media format (codec) for reading on the specified line.
      */
    virtual OpalMediaFormat GetReadFormat(
      unsigned line    /// Number of line
    );

    /**Get the media format (codec) for writing on the specified line.
      */
    virtual OpalMediaFormat GetWriteFormat(
      unsigned line    /// Number of line
    );
   /**Get the read frame size in bytes.
       All calls to ReadFrame() will return this number of bytes.
      */
    virtual PINDEX GetReadFrameSize(
      unsigned line   /// Number of line
    );

    virtual BOOL SetReadFrameSize(unsigned, PINDEX);

    /**Get the write frame size in bytes.
       All calls to WriteFrame() must be this number of bytes.
      */
    virtual PINDEX GetWriteFrameSize(
      unsigned line   /// Number of line
    );

    virtual BOOL SetWriteFrameSize(unsigned, PINDEX);

    /**Low level read of a frame from the device.
     */
    virtual BOOL ReadFrame(
      unsigned line,    /// Number of line
      void * buf,       /// Pointer to a block of memory to receive data.
      PINDEX & count    /// Number of bytes read, <= GetReadFrameSize()
    );

    /**Low level write frame to the device.
     */
    virtual BOOL WriteFrame(
      unsigned line,    /// Number of line
      const void * buf, /// Pointer to a block of memory to write.
      PINDEX count,     /// Number of bytes to write, <= GetWriteFrameSize()
      PINDEX & written  /// Number of bytes written, <= GetWriteFrameSize()
    );

    virtual BOOL PlayDTMF(
      unsigned line,            /// Number of line
      const char * digits,      /// DTMF digits to be played
      DWORD onTime = DefaultDTMFOnTime,  /// Number of milliseconds to play each DTMF digit
      DWORD offTime = DefaultDTMFOffTime /// Number of milliseconds between digits
    );

    virtual char ReadDTMF(unsigned line);

    BOOL SetAEC(
      unsigned line,    /// Number of line
      AECLevels level  /// AEC level
        );

    BOOL SetVAD(
      unsigned line,    /// Number of line
      BOOL enable       /// Flag for enabling VAD
    );

    int decide_delay(int incoming, int *framecnt);
    unsigned WaitForToneDetect(unsigned line, int timeout);

    OpalLineInterfaceDevice & GetDevice() const { return *device; }
    unsigned GetLineNumber() const { return 0; }
    
    CodecXRefInfo *isValidFormat(const char *o);

    int  in_min;
    int  in_max;
    int  out_min;
    int  out_max;

  private:

//    unsigned int	lineId;
    PString  	devname;
    OpalLineInterfaceDevice *device;
    int  writesem;
    int  outframecnt;
    int  inframecnt;
    //time_t old_in_clk;
    //time_t old_out_clk;
    timeval old_in_clk;
    timeval old_out_clk;
    BOOL readStopped,writeStopped;
//    GatewayEndPoint   & endpoint;
};

class GatewayConnection : public H323Connection
{
  PCLASSINFO(GatewayConnection, H323Connection);

  public:
    GatewayConnection(GatewayEndPoint &, unsigned);
    ~GatewayConnection();

    void OnUserInputString(const PString & value);
   /// Session with statistics

    void Remove_caps();
    H323Capability *SelectLocalCapabilities(H323Capability &); 
    void OnCleared();
    void OnSelectLogicalChannels();
    void SelectFastStartChannels(unsigned sessionID,
					BOOL transmitter,
						BOOL receiver);
    BOOL OnStartLogicalChannel(H323Channel & channel);

    virtual void OnLogicalChannelJitter(
      H323Channel * channel,   /// Channel that is to be limited
      DWORD jitter,            /// Estimated received jitter in microseconds
      int skippedFrameCount,   /// Frames skipped by decodec
      int additionalBuffer     /// Additional size of video decoder buffer
    );


    BOOL CheckFastStart(void);
    int  SetPnxCaps(const H323Capabilities & Caps);
//    int  SetPnxCaps(const H245_OpenLogicalChannel & open);
    void StartFastStartChannel(
                H323LogicalChannelList & fastStartChannels,
                   unsigned sessionID, H323Channel::Directions direction);


    OpalPnXDevice   	*line;
    PString	gwcallToken;
    PString	h323Address;
    struct ast_channel  *ast_chan;
    H323Channel		*rx_chan;
    H323Channel		*tx_chan;
    struct h323_pvt	*h323pvt;
    H323Connection::AnswerCallResponse  callState;
    H323TransportAddress remote_endpoint;
};

class PNXGw : public PProcess
{
  PCLASSINFO(PNXGw, PProcess)

  public:
    PNXGw();
    ~PNXGw();

    void Main();
};


#endif  // 


// End of File ///////////////////////////////////////////////////////////////

