/*
 * pnxcodec.h
 *
 */

#ifndef __PNX_CODEC_H
#define __PNX_CODEC_H

#include <lid.h>
#include "h323caps.h"

struct CodecXRefInfo
{
     const   char *h323_codec_name;
     unsigned int    PnxCodecId;
     H245_AudioCapability::Choices capabilitySubtype;
     int  isFramedCodec;
     int  framesz;
     int  rxframes;
     int  txframes;
     unsigned rxFramesInPacket;
     unsigned txFramesInPacket;
     int  in_min;
     int  in_max;
     int  out_min;
     int  out_max;
     int  preference;
};


/*
struct PnxCodecInfo 
{
   int  rxFramesInPacket;
   int  txFramesInPacket;
   int  silenceSuppression;
   int  canSilenceSuppression;
   int  samplesPerFrame;
   int  bits;
   int  typeFramedAudio;
   char h323_codec_name[64];
   int pnxCodecId;
   H245_AudioCapability::Choices capabilitySubtype;    
   unsigned maxAl_sduAudioFrames;
};
*/

///////////////////////////////////////////////////////////////////////////////

/**This class describes the PNX codec capability.
 */
class H323_PnxCapability : public H323_LIDCapability
{
  PCLASSINFO(H323_PnxCapability, H323_LIDCapability)

  public:

    /**Create a new PNX capability.  */

    H323_PnxCapability();
    H323_PnxCapability(int i);
    ~H323_PnxCapability();

    int Priority();

    /**Create the codec instance, allocating resources as required.
     */

    H323Codec * CreateCodec( H323Codec::Direction direction 
    ) const;

  /**@name Overrides from class PObject */

    /**Create a copy of the object.
      */
  //  PObject * Clone() const;

  /* name Operations */

  /** name Identification functions */

    /**Get the sub-type of the capability. This is a code dependent on the
       main type of the capability.

       This returns one of the four possible combinations of mode and speed
       using the enum values of the protocol ASN H245_AudioCapability class.
     */
     // unsigned GetSubType() const;

    /**Get the name of the media data format this class represents.
     */
      PString GetFormatName() const;

    /**Set the maximum size (in frames) of data that will be transmitted in a
       single PDU.

       This will also be the desired number that will be sent by most codec
       implemetations.

       The default behaviour sets the txFramesInPacket variable.
//    virtual void SetTxFramesInPacket(
//      unsigned frames   /// Number of frames per packet
//    );
     */

  /**@name Protocol manipulation */

    /**This function is called whenever and outgoing TerminalCapabilitySet
       or OpenLogicalChannel PDU is being constructed for the control channel.
       It allows the capability to set the PDU fields from information in
       members specific to the class.

       The default behaviour sets the data rate field in the PDU.
     */
/*
    BOOL OnSendingPDU(
      H245_AudioCapability & pdu,  /// PDU to set information on
      unsigned packetSize          /// Packet size to use in capability
    ) const;
*/
    /**This function is called whenever and incoming TerminalCapabilitySet
       or OpenLogicalChannel PDU has been used to construct the control
       channel. It allows the capability to set from the PDU fields,
       information in members specific to the class.

       The default behaviour gets the data rate field from the PDU.
     */
/*
    BOOL OnReceivedPDU(
      const H245_AudioCapability & pdu,  /// PDU to get information from
      unsigned & packetSize              /// Packet size to use in capability
    );
*/
 
//  protected:
    CodecXRefInfo *pci;
};

/////////////////////////////////////////////////////////////////////////////

#endif // __PNX_CODEC_H


/////////////////////////////////////////////////////////////////////////////

