// chan_h323_pnx.cxx
/*
 * H.323 protocol handler
 *
 * Copyright (c) 2002-2003 File-Sure Pte. Ltd.
 *
 * The contents of this file are distributed under the terms of
 * the GNU General Public License
 *
 * Software distributed under this License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Original Code is File-Sure Pte. Ltd.
 * -----------------------------------------------------------------
 * $Log: chan_h323_pnx.cxx,v $
 * Revision 1.4  2002/05/18 01:08:36  benjamin
 * This release tested with Zapata XP100 FXO card.
 *
 * Revision 1.3  2002/04/16 09:23:41  benjamin
 * This records another merge with lastest Asterisk CVS.
 * Some new files still missing.
 *
*/
 
#if 0
#include <ptlib.h>

#include <h323pdu.h>
#include <h323con.h>
#include <gsmcodec.h>
#include <lid.h>

#ifdef HAS_VPB
#include <vpblid.h>
#ifndef __GNUC__
#pragma message("Including VPB support.")
#endif
#endif

#include "chan_h323.h"
#endif

static int dbg = 0;       // set to greater than 2 to off
// static int dbgfp = -1;
int decide_delay(int a, int *b);

static int memcnt(const unsigned char *b, unsigned char ch, int count)
{
    int i,c;
    for (i=0,c=0; i < count; i++)
       if (b[i] == ch) c++;
    return c;
}
 

BOOL OpalPnXDevice::Open(const PString & dev)
{
  devname = dev;
  device = this;
  writesem = pvt->writesem;
  inframecnt = 0;
  outframecnt = 0;
  timerclear(&old_in_clk);
  timerclear(&old_out_clk);
  readStopped = writeStopped = TRUE;
  if (debugtrace > 1)
     cout << "OpalPnXDevice Open " << dev << endl;

  return TRUE;
}

BOOL OpalPnXDevice::Close()
{
   if (debugtrace > 1)
     cout << "OpalPnXDevice Close=" << devname << endl;

   if (pvt != (struct h323_pvt *)NULL)
   {
      pvt->isactive = FALSE;
      pvt->obuflen = 0;
      pvt->ibuflen = 0;
      pthread_mutex_unlock(&pvt->watermark);
      PThread::Current()->Sleep(2);
      if (debugtrace > 2)
         cout << "OpalPnXDevice Close Pipe" << endl;
      close(pvt->infd[1]);
      close(pvt->infd[0]);
      if (debugtrace > 2)
         cout << "OpalPnXDevice Close Pipe Done" << endl;
   }

//   if (dbgfp != -1)
//     close(dbgfp);
//   dbgfp = -1;

   pvt = NULL;

   return TRUE;
};

BOOL OpalPnXDevice::StopReadCodec(unsigned line)
{
//   struct ast_channel  *chan;
   if (debugtrace > 1)
      cout << "OpalPnXDevice StopReadCodec" << devname << endl;
   if ((pvt != NULL && pvt->isactive) && !readStopped) 
   {
       if (debugtrace > 2)
         cout << "StopReadCodec 2" << endl;
       V(writesem);
       pvt->isactive = FALSE;

       if (debugtrace > 2)
          cout << "OpalPnXDevice StopReadCodec 1" << endl;
   }

   if (debugtrace > 1)
      cout << "OpalPnXDevice StopReadCodec 2" << endl;
   readStopped = TRUE;
   return OpalLineInterfaceDevice::StopReadCodec(line);
}

BOOL OpalPnXDevice::StopWriteCodec(unsigned line)
{
   struct ast_channel  *chan;

   if (debugtrace > 1)
      cout << "OpalPnXDevice StopWriteCodec " << devname << endl;
   if ((pvt != NULL && pvt->isactive) && !writeStopped) 
   {
       if (debugtrace > 3)
          cout << "OpalPnXDevice StopWriteCodec 1" << endl;
       pvt->isactive = FALSE;
       if (!readStopped)
       {
          readStopped = TRUE;
          V(writesem);
       }

       if (debugtrace > 3)
          cout << "OpalPnXDevice StopWriteCodec 2" << endl;
   }

   if (debugtrace > 1)
      cout << "OpalPnXDevice StopWriteCodec 3" << endl;
   writeStopped = TRUE;
   return OpalLineInterfaceDevice::StopWriteCodec(line);
}
 
unsigned OpalPnXDevice::GetLineCount()
{
  if (debugtrace > 1)
    cout << "OpalPnXDevice::GetLineCount" << endl;
  return 1;
}

PString OpalPnXDevice::GetName() const
{
  return "PNX " + devname;
}

BOOL OpalPnXDevice::IsLineOffHook(unsigned line)
{
   int r = FALSE;

   if (debugtrace > 1)
      cout << "OpalPnXDevice::IsLineOffHook" << endl;
   switch (pvt->owner->state)
   {
	case AST_STATE_OFFHOOK:
	case AST_STATE_UP:
	case AST_STATE_BUSY:
	case AST_STATE_DIALING:
		r = TRUE;
   }
   return (r);
}

BOOL OpalPnXDevice::SetLineOffHook(unsigned line,    // Number of line
      				BOOL newState  	     // New state to set
			   )
{
   if (debugtrace > 1)
     cout << "OpalPnXDevice::SetLineOffHook" << endl;
   return FALSE;
}

OpalMediaFormat::List OpalPnXDevice::GetMediaFormats() const
{
    OpalMediaFormat::List codecs;

    if (debugtrace > 1)
       cout << "OpalMediaFormat::List" << endl;
    codecs.Append(new OpalMediaFormat(OPAL_PCM16));
    return codecs;
}


unsigned OpalPnXDevice::WaitForToneDetect(unsigned line, int timeout)
{
   unsigned r = 9999;
  
   if (debugtrace > 1)
      cout << "WaitForToneDetect" << endl;
   switch (pvt->owner->state)
   {
	case AST_STATE_RING:
	case AST_STATE_RINGING:
                if (debugtrace > 1)
		   cout << "RingTone" << endl;
		r = OpalLineInterfaceDevice::RingTone;
		break;
	case AST_STATE_BUSY:
		r = OpalLineInterfaceDevice::BusyTone;
		break;
	case AST_STATE_DOWN:
	case AST_STATE_RESERVED:
		r = OpalLineInterfaceDevice::NoTone;
		break;
   }
   return r;
}

CodecXRefInfo *OpalPnXDevice::isValidFormat(const char *mediaFmt)
{
    extern struct CodecXRefInfo CodecXRInfo[];
    //char   s[32];

    for (int x=0; CodecXRInfo[x].h323_codec_name != NULL; x++)
    {
         if (strncmp(mediaFmt,CodecXRInfo[x].h323_codec_name,
                 strlen(CodecXRInfo[x].h323_codec_name)) == 0)
         {
            //cout << "isValidFormat found " << mediaFmt << endl;
	    return &CodecXRInfo[x];
         }
    }
    return (CodecXRefInfo *)NULL;
}

        
BOOL OpalPnXDevice::SetReadFormat(unsigned line,
     const OpalMediaFormat & mediaFormat  )
{
//     cout << endl;
//     cout << "SetReadFormat-->" << mediaFormat << endl;
     {
         CodecXRefInfo *xf;

         // maybe its because the hw capabilities is set
         if ((xf=isValidFormat(mediaFormat)) != NULL)
         {
            pvt->lastoutput = xf->PnxCodecId;
          //   pvt->rx_frlen = xf->rxframes;
            pvt->tx_frlen = xf->framesz;  //xf->txframes;
            pvt->tx_codec = xf;
	    pvt->tx_framerate = xf->rxframes; // xf->txframes;  //xf->framesz;
            pvt->out_framerate = 0;
            readStopped = FALSE;
	    pvt->maxreadlen = pvt->tx_frlen * NBUF;
	    out_min = xf->out_min;
	    out_max = xf->out_max;
            if (debugtrace > 1)
               cout << endl << "SetReadFormat-->" << mediaFormat << endl;
	    return TRUE;
         }
     }
     return FALSE;
}

BOOL OpalPnXDevice::SetWriteFormat( unsigned line,  
      const OpalMediaFormat & mediaFormat)
{
//     cout << "SetWriteFormat" << endl;
     {
         CodecXRefInfo *xf;

         // maybe its because the hw capabilities is set
         if ((xf=isValidFormat(mediaFormat)) != NULL)
         {
            pvt->lastinput = xf->PnxCodecId;
            pvt->rx_frlen = xf->framesz;       //xf->rxframes;
            //pvt->tx_frlen = xf->txframes;
            pvt->rx_codec = xf;
            pvt->rx_framerate = xf->txframes;   //xf->framesz;
	    pvt->in_framerate = 0;
	    writeStopped = FALSE;
            in_min = xf->in_min;
	    in_max = xf->in_max;
	    pvt->maxwritelen = pvt->rx_frlen * 6;
            if (debugtrace > 1)
               cout << endl << "SetWriteFormat-->" << mediaFormat << endl;
            return TRUE;
         }
     }
     return FALSE;
}

OpalMediaFormat OpalPnXDevice::GetReadFormat( unsigned line )
{
     if (pvt->tx_codec == NULL)
	return(OPAL_PCM16);
     else
       return(pvt->tx_codec->h323_codec_name);
}

OpalMediaFormat OpalPnXDevice::GetWriteFormat(
          unsigned line  )
{
     if (pvt->rx_codec == NULL)
	return(OPAL_PCM16);
     else
       return(pvt->rx_codec->h323_codec_name);
}

PINDEX OpalPnXDevice::GetReadFrameSize(
      unsigned line )
{
//	cout << "OpalPnXDevice::GetReadFrameSize" << endl;
	return pvt->tx_frlen;
}

BOOL OpalPnXDevice::SetReadFrameSize(unsigned line, PINDEX j)
{
//	cout << "OpalPnXDevice::SetReadFrameSize to " << j << endl;
	return FALSE;
}

PINDEX OpalPnXDevice::GetWriteFrameSize(
      unsigned line  )
{
//	cout << "OpalPnXDevice::GetWriteFrameSize" << endl;
	return pvt->rx_frlen;
}

BOOL OpalPnXDevice::SetWriteFrameSize(unsigned line, PINDEX i)
{
//	cout << "OpalPnXDevice::SetWriteFrameSize to " << i << endl;
	return FALSE;
}


BOOL OpalPnXDevice::ReadFrame( unsigned line,
	 void * buf,      
	 PINDEX & count )
{
    int readLen = pvt->tx_frlen;
    void *readBuf = buf;

    if (debugtrace > 2 && dbg < 3)
    {
       ++dbg;   
       cout << "ReadFrame 1 len=" << readLen << endl;
      //dbgfp = open("/tmp/dbg.h323",O_WRONLY|O_TRUNC,0666);
    }

    if (readStopped)
    {
       if (debugtrace > 2 )
          cout << "ReadFrame Stopped" << endl;
       pthread_mutex_unlock(&pvt->watermark);
       return FALSE;
    }

    if (pvt == (struct h323_pvt *)NULL || !pvt->isactive)
    {
        if (debugtrace > 2 )
	   cout << "Bad PVT active=" << pvt->isactive << endl;
        pthread_mutex_unlock(&pvt->watermark);
        return FALSE;
    }

    P(writesem);
    
    if (readStopped || pvt == (struct h323_pvt *)NULL || !pvt->isactive)
    {
        if (debugtrace > 2 )
        {
	   cout << "Bad PVT " << devname << endl;
           cout << "readStopped=" << readStopped << "isactive=" << pvt->isactive << endl;
	}
        pthread_mutex_unlock(&pvt->watermark);
        return FALSE;
    }
  
//    if (pvt->in_buffered > 0)
//           sleep(1);

    outframecnt++;
    PThread::Current()->Sleep(decide_delay(0,&outframecnt));
    //pvt->writewait = decide_delay(0,&outframecnt);
    pthread_mutex_lock(&pvt->writelock);
       
    if (pvt->obuflen >= readLen)
    {
        memcpy(readBuf,(const void *)pvt->obuf,readLen);
        //write(dbgfp,readBuf,readLen);
	//ioctl(dbgfp,FDFLUSH);
	pvt->frames_buffered--;
        pvt->obuflen -= readLen;
        if (pvt->obuflen < readLen)
        {
             pvt->obuflen = 0;
	     pvt->frames_buffered = 0;
	}

        /* Move memory if necessary */
        if (pvt->obuflen >= 0 )
              memmove(pvt->obuf, pvt->obuf + readLen, pvt->obuflen);
        count = readLen;
    }
    else 
    {
        pvt->obuflen = 0;
	pvt->frames_buffered = 0;
	count = 0;
    }
 
read_exit: 
    if ((pvt->maxreadlen-pvt->obuflen) >= (pvt->maxreadlen/2))
            pthread_mutex_unlock(&pvt->watermark);

    pthread_mutex_unlock(&pvt->writelock);

    if (debugtrace > 3)
       cout << "ReadFrame Ended" << endl;
    return TRUE;
}


/**Low level write frame to the device.
 */
BOOL OpalPnXDevice::WriteFrame(
      unsigned line,   
      const void * buf,
      PINDEX count,  
      PINDEX & written )
{
//    void *writeBuf = (void *)buf;

    int  len;
    int  space, nframes;
    int  frlen = 480;
    int  x;

    if (debugtrace > 2 && dbg < 2)
    {
      char fn[20];
       dbg++;   
       cout << "WriteFrame" << endl;
       sprintf(fn,"/tmp/dbg-%d.fl",dbg);
 //      dbgfp = open(fn,O_CREAT|O_WRONLY|O_TRUNC,0666);
    }


    if (writeStopped || pvt == (struct h323_pvt *)NULL || !pvt->isactive)
    {
        if (debugtrace > 2)
           cout << "WriteFrame Stopped" << endl;
	return FALSE;
    }

    frlen = len = pvt->rx_frlen;

    if ((int)count != len)
    {
	written = count;	// eat not okay size frame
	return TRUE;

	// len = count;
    }
  
    written = len;

    if ( memcnt((const unsigned char *)buf,0,len) == len )
       return TRUE;   // skip silence frame

 
    if (pvt == (struct h323_pvt *)NULL || !pvt->isactive)
    {
	return FALSE;
    }
  
//    write(dbgfp,buf,len);
//    ioctl(dbgfp,FDFLUSH);

    pthread_mutex_lock(&pvt->readlock);

    if (pvt == (struct h323_pvt *)NULL || !pvt->isactive )
    {
        pthread_mutex_unlock(&pvt->readlock);
	return FALSE;
    }
    
    space = pvt->maxwritelen - pvt->ibuflen;

    if (space < len)
    {
        pthread_mutex_unlock(&pvt->readlock);
        while (pvt != NULL && pvt->isactive && space < len)
	{
     	   if (write(pvt->infd[1],"WRITE\n\0",6) < 0)
           {
	        if (debugtrace > 3)
	      	   cout << "WriteFrame: pipe full" << endl;
                return TRUE;
	   }
           ioctl(pvt->infd[1],FDFLUSH);
	   if (debugtrace > 4)
    		cout << "WriteFrame:: buffer FULL" << endl;
           PThread::Current()->Sleep( decide_delay(1,&inframecnt)+5 );
           space =  pvt->maxwritelen - pvt->ibuflen;
        }

        if (pvt == (struct h323_pvt *)NULL || !pvt->isactive )
        {
           return FALSE;
        }

        pthread_mutex_lock(&pvt->readlock);

    }

    memcpy(pvt->ibuf + pvt->ibuflen, buf, len);
    pvt->ibuflen += len;

    pvt->in_buffered++;

     write(pvt->infd[1],"WRITE\n\0",6); 
     ioctl(pvt->infd[1],FDFLUSH);
     
write_exit:

    pthread_mutex_unlock(&pvt->readlock);

    inframecnt++;
    PThread::Current()->Sleep( decide_delay(1,&inframecnt)+1 );

    if (debugtrace > 3)
       cout << "WriteFrame Return" << endl;
     return TRUE;
}

BOOL OpalPnXDevice::PlayDTMF(
      unsigned line,            /// Number of line
      const char * digits,      /// DTMF digits to be played
      DWORD onTime = DefaultDTMFOnTime,  /// Number of milliseconds to play each DTMF digit
      DWORD offTime = DefaultDTMFOffTime /// Number of milliseconds between digits
    )
{
   if (debugtrace > 1)
     cout << "PlayDTMF: " << digits << endl;
   
   if (strlen(pvt->dtmf_str) < AST_MAX_EXTENSION)
   {
      strcat(pvt->dtmf_str,digits);
      pvt->dtmf_ontime = onTime;
      pvt->dtmf_offtime = offTime;
   }
   return TRUE;
}

char OpalPnXDevice::ReadDTMF(unsigned line)
{
   if (debugtrace > 1)
    cout << "ReadDTMF" << endl;
    return FALSE;
}

BOOL OpalPnXDevice::SetAEC( unsigned line, AECLevels level )
{
   if (debugtrace > 1)
     cout << "SetAEC" << endl;
     return FALSE;
}

BOOL OpalPnXDevice::SetVAD( unsigned line,  BOOL enable )
{
   if (debugtrace > 1)
    cout << "SetVAD" << endl;
    return FALSE;
}


